import { __name } from './chunk-O6YSETKJ.mjs';
import { Cursor } from './cursor';

async function cursorPaginationFind(em, entityName, pagination, where = {}, options = {}) {
  if ("after" in pagination && pagination.after && "before" in pagination && pagination.before) {
    throw new Error(`Cannot use both 'after' and 'before' at the same time`);
  }
  if ("first" in pagination && pagination.first && "last" in pagination && pagination.last) {
    throw new Error(`Cannot use both 'first' and 'last' at the same time`);
  }
  if ("first" in pagination && pagination.first < 0) {
    throw new Error(`'first' must be greater than or equal to 0`);
  }
  if ("last" in pagination && pagination.last < 0) {
    throw new Error(`'last' must be greater than or equal to 0`);
  }
  const noCursorPagination = "orderBy" in pagination;
  if (noCursorPagination) {
    const [totalCount, entities] = await Promise.all([
      em.count(entityName, {
        ...where
      }),
      em.find(entityName, {
        ...where
      }, {
        ...options,
        limit: pagination.first,
        orderBy: pagination.orderBy
      })
    ]);
    const edges = entities.map((entity) => {
      return {
        cursor: Cursor.fromEntity(entity, pagination.orderBy).toBase64(),
        node: entity
      };
    });
    return {
      totalCount,
      edges,
      pageInfo: {
        hasNextPage: totalCount > pagination.first,
        hasPreviousPage: false,
        startCursor: edges[0]?.cursor,
        endCursor: edges[edges.length - 1]?.cursor
      }
    };
  }
  const forwardPagination = "after" in pagination;
  if (forwardPagination) {
    const cursor = Cursor.fromBase64(pagination.after);
    const paginationWhere = {
      $and: [
        {
          ...where
        },
        {
          $or: cursor.buildWhereOr({
            direction: "forward"
          })
        }
      ]
    };
    const paginationOptions = {
      ...options,
      limit: pagination.first + 1,
      orderBy: cursor.buildOrderBy({
        direction: "forward"
      })
    };
    const [totalCount, entities] = await Promise.all([
      em.count(entityName, where),
      em.find(entityName, paginationWhere, paginationOptions)
    ]);
    let hasNextPage = false;
    if (entities.length === pagination.first + 1) {
      hasNextPage = true;
      entities.pop();
    }
    const orderBy = cursor.buildOrderBy({
      direction: "forward"
    });
    const edges = entities.map((entity) => {
      return {
        cursor: Cursor.fromEntity(entity, orderBy).toBase64(),
        node: entity
      };
    });
    return {
      totalCount,
      edges,
      pageInfo: {
        hasNextPage,
        hasPreviousPage: true,
        startCursor: edges[0]?.cursor,
        endCursor: edges[edges.length - 1]?.cursor
      }
    };
  }
  const backwardPagination = "before" in pagination;
  if (backwardPagination) {
    const cursor = Cursor.fromBase64(pagination.before);
    const paginationWhere = {
      $and: [
        {
          ...where
        },
        {
          $or: cursor.buildWhereOr({
            direction: "backward"
          })
        }
      ]
    };
    const paginationOptions = {
      ...options,
      limit: pagination.last + 1,
      orderBy: cursor.buildOrderBy({
        direction: "backward"
      })
    };
    const [totalCount, entities] = await Promise.all([
      em.count(entityName, where),
      em.find(entityName, paginationWhere, paginationOptions)
    ]);
    let hasPreviousPage = false;
    if (entities.length === pagination.last + 1) {
      hasPreviousPage = true;
      entities.pop();
    }
    const orderBy = cursor.buildOrderBy({
      direction: "forward"
    });
    const edges = entities.reverse().map((entity) => {
      return {
        cursor: Cursor.fromEntity(entity, orderBy).toBase64(),
        node: entity
      };
    });
    return {
      totalCount,
      edges,
      pageInfo: {
        hasNextPage: true,
        hasPreviousPage,
        startCursor: edges[0]?.cursor,
        endCursor: edges[edges.length - 1]?.cursor
      }
    };
  }
  throw new Error("Invalid pagination parameters");
}
__name(cursorPaginationFind, "cursorPaginationFind");

export { cursorPaginationFind };
